/*
		==     =                    *  =      =
		| -    |                        -    - 
		|  -   |  =   =  =--=  =    =    ====  
		|   -  |  |   |  |     |    |    ====  
		|    - |  |   |  |     |    |   -    - 
		=     ==  =---=  =--=  =--= =  =      =
*/

#ifndef _HEAP_H
#define _HEAP_H

#include "OrderedList.h"
#include "Paging.h"

/* Defines a heap header (beginning of a block). */
struct HeapHeader
{
	unsigned Magic;										/* Magic identification number. */
	bool IsHole						: 1;				/* Is this header at the start of a hole? */
	unsigned Size;										/* The size of the block (footer end inclusive). */
};

/* Defines a heap footer (end of a block). */
struct HeapFooter
{
	unsigned Magic;										/* Magic identification number. */
	HeapHeader *Header;									/* Pointer to the header that goes with this footer. */
};

/* General structure to define the heap itself. */
struct Heap
{
	OrderedList Index;									/* Keeps track of all the blocks. */

	unsigned StartAddress;								/* Allocated memory start. */
	unsigned EndAddress;								/* Allocated memory end. */
	unsigned MaxAddress;								/* The address that can be (maximally) expanded to. */

	bool SupervisorOnlyNewPages		: 1;				/* Make new pages supervisor only? */
	bool ReadOnlyNewPages			: 1;				/* Make new pages read only? */
};

Heap *CreateHeap(unsigned StartAddress, unsigned		/* Creates a new heap. */
	EndAddress, unsigned MaxAddress, bool SupervisorOnly,
	bool ReadOnly);

void *HeapAlloc(unsigned Size, bool PageAlign,			/* Allocates memory. */
	Heap *_Heap);
void HeapFree(void *Memory, Heap *_Heap);				/* Frees up heap memory allocated by HeapAlloc. */

#endif //_HEAP_H
