#include "OrderedList.h"

OrderedList CreateOrderedList(unsigned MaxSize, bool (*IsLessThanFunc) (void *a, void *b))
{
	OrderedList Instance;

	/* Fill in the values. */
	Instance.Array = reinterpret_cast<void **>(Memory->Alloc(MaxSize * sizeof(void *)));
	Instance.Size = 0;
	Instance.MaxSize = MaxSize;
	Instance.IsLessThan = IsLessThanFunc;

	/* Set all the values inside the array to 0. */
	Memory->Set(Instance.Array, 0, MaxSize * sizeof(void *));

	return Instance;
}

OrderedList PutOrderedList(void *Address, unsigned MaxSize, bool (*IsLessThanFunc) (void *a, void *b))
{
	OrderedList Instance;

	/* Fill in the values. */
	Instance.Array = reinterpret_cast<void **>(Address);
	Instance.Size = 0;
	Instance.MaxSize = MaxSize;
	Instance.IsLessThan = IsLessThanFunc;

	/* Set all the values inside the array to 0. */
	Memory->Set(Instance.Array, 0, MaxSize * sizeof(void *));

	return Instance;
}

void FreeOrderedList(OrderedList *List)
{
	Memory->Free(List->Array);
}

bool InsertListItem(OrderedList *List, void *Item)
{
	if(!List->IsLessThan)
		return false;

	unsigned i = 0;

	while(i < List->Size && List->IsLessThan(&List->Array[i], Item))
		++i;

	/* Have we reached the end of the list? */
	if(i == List->Size)
	{
		/* Put the item at the end of the array (actually outside) and increment the size. */
		List->Array[List->Size++] = Item;
	}

	else
	{
		void *OldItem = &List->Array[i];
		List->Array[i] = Item;

		while(i < List->Size)
		{
			void *Old = List->Array[++i];
			List->Array[i] = OldItem;
			OldItem = Old;
		}

		++List->Size;
	}

	return true;
}

void *RetrieveListItem(OrderedList *List, unsigned Id)
{
	if(Id >= List->Size)
		return 0;

	return List->Array[Id];
}

void DeleteListItem(OrderedList *List, unsigned Id)
{
	while(Id < List->Size)
	{
		List->Array[Id] = List->Array[Id + 1];
		++Id;
	}

	--List->Size;
}
