/*
		==     =                    *  =      =
		| -    |                        -    - 
		|  -   |  =   =  =--=  =    =    ====  
		|   -  |  |   |  |     |    |    ====  
		|    - |  |   |  |     |    |   -    - 
		=     ==  =---=  =--=  =--= =  =      =
*/

#ifndef _ORDERED_LIST_H
#define _ORDERED_LIST_H

#include "../Defines.h"

#include "Memory.h"

/* Structure harnassing an ordered list. */
struct OrderedList
{
	void **Array;										/* The actual array. */

	unsigned Size;										/* Size of the array. */
	unsigned MaxSize;									/* Maximum size of the array. */

	bool (*IsLessThan) (void *a, void *b);				/* Function pointer to the function to use to determine smallest pointers. */
};

OrderedList CreateOrderedList(unsigned MaxSize,			/* Creates an ordered list from the specified parameters. */
	bool (*IsLessThanFunc) (void *a, void *b));

OrderedList PutOrderedList(void *Address, unsigned		/* Places a new ordered list at the specified memory address. */
	MaxSize, bool (*IsLessThanFunc) (void *a, void *b));

void FreeOrderedList(OrderedList *List);				/* Frees and destroys an ordered list. */

bool InsertListItem(OrderedList *List, void *Item);		/* Inserts an item into the specified ordered list. */
void *RetrieveListItem(OrderedList *List, unsigned Id);	/* Retrieves the item at the specified index. */
void DeleteListItem(OrderedList *List, unsigned Id);	/* Deletes the item at the specified index. */

#endif //_ORDERED_LIST
